# -*- coding: utf-8 -*-

"""A getter for HGNC."""

import ftplib

from ..utils import Getter, VersionType

__all__ = [
    "HGNCGetter",
]

HOST = "ftp.ebi.ac.uk"
PATH = "pub/databases/genenames/hgnc/archive/monthly/json/"
PREFIX = "hgnc_complete_set_"
SUFFIX = ".json"


class HGNCGetter(Getter):
    """A getter for HGNC."""

    bioregistry_id = "hgnc"
    name = "HGNC"
    homepage_fmt = (
        "https://ftp.ebi.ac.uk/pub/databases/genenames/hgnc/"
        "archive/monthly/json/hgnc_complete_set_{version}.json"
    )

    version_type = VersionType.date

    def get(self) -> str:
        """Get the latest HGNC version number."""
        with ftplib.FTP(HOST) as ftp:
            ftp.login()
            ftp.cwd(PATH)
            version = max(
                name[len(PREFIX) : -len(SUFFIX)] for name in ftp.nlst() if name.startswith(PREFIX)
            )
        return version


if __name__ == "__main__":
    HGNCGetter.print()



# -*- coding: utf-8 -*-

"""A getter for Ensembl."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "EnsemblGetter",
]

URL = "https://useast.ensembl.org/index.html"


class EnsemblGetter(Getter):
    """A getter for DrugBank."""

    bioregistry_id = "ensembl"
    name = "Ensembl"
    homepage_fmt = "https://www.ensembl.org"
    date_fmt = "%b %Y"
    version_type = VersionType.sequential

    def get(self):
        """Get the latest Ensembl version number."""
        soup = get_soup(URL, verify=False)
        manifest = soup.find(**{"class": "box-header"}).text
        version, date = manifest.rstrip(")").split("(", 1)
        return dict(version=version.split()[-1], date=date)


if __name__ == "__main__":
    EnsemblGetter.print()



# -*- coding: utf-8 -*-

"""A getter for MSigDB."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "MSigDBGetter",
]

URL = "http://www.gsea-msigdb.org/gsea/msigdb/index.jsp"


class MSigDBGetter(Getter):
    """A getter for MSigDB."""

    bioregistry_id = "msigdb"
    name = "MSigDB"
    homepage_fmt = "https://data.broadinstitute.org/gsea-msigdb/msigdb/release/{version}"
    version_type = VersionType.year_minor

    def get(self):
        """Get the latest MSigDB version number."""
        soup = get_soup(URL)

        x = soup.find(text="Current Version")
        paragraph = x.parent.find_next_sibling("p")
        version = paragraph.text.strip().split()[2][len("v"):-len(".Hs")]
        return version


if __name__ == "__main__":
    MSigDBGetter.print()



# -*- coding: utf-8 -*-

"""A getter for Biofacquim."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "BiofacquimGetter",
]


class BiofacquimGetter(Getter):
    """A getter for Biofacquim."""

    name = "Biofacquim"
    version_type = VersionType.semver_minor

    def get(self) -> str:
        """Get the latest NPASS version number."""
        soup = get_soup("https://biofacquim.herokuapp.com/")
        h3 = soup.find(name="h3")
        return h3.text[len("version ") :]


if __name__ == "__main__":
    BiofacquimGetter.print()



# -*- coding: utf-8 -*-

"""A getter for the NCI Thesaurus."""

import re
from typing import Dict

from ..utils import Getter, VersionType, get_soup

__all__ = [
    "NCItGetter",
]

URL = "https://ncithesaurus.nci.nih.gov/ncitbrowser/"
PATTERN = re.compile(
    r"Version:([0-9]{2}\.[0-9]{2}[a-z]) " r"\(Release date:([0-9]{4}-[0-9]{2}-[0-9]{2})"
)


class NCItGetter(Getter):
    """A getter for the NCI Thesaurus."""

    bioregistry_id = "ncit"
    name = "National Cancer Institute Thesaurus"
    date_fmt = "%Y-%m-%d"
    version_type = VersionType.other

    def get(self) -> Dict[str, str]:
        """Get the latest NCIt version number."""
        soup = get_soup(URL)
        version_str = soup.find("span", {"class": "vocabularynamelong_ncit"}).contents[0]
        match = re.search(PATTERN, version_str)
        if match is None:
            raise ValueError(f"could not parse version from {URL}")
        return {
            "version": match.group(1),
            "date": match.group(2),
        }


if __name__ == "__main__":
    NCItGetter.print()



# -*- coding: utf-8 -*-

"""A getter for the Antibody Registry."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "AntibodyRegistryGetter",
]


class AntibodyRegistryGetter(Getter):
    """A getter for the Antibody Registry."""

    bioregistry_id = "antibodyregistry"
    name = "Antibody Registry"
    homepage_fmt = "https://antibodyregistry.org/"
    version_type = VersionType.semver_minor

    def get(self):
        """Get the latest Antibody Registry version number."""
        soup = get_soup("https://antibodyregistry.org/")
        return soup.find(**{"class": "footer"}).find("a").text.lstrip("v")


if __name__ == "__main__":
    AntibodyRegistryGetter.print()
